--- /usr/src/asterisk/asterisk-16.21.1.orig/res/res_pjsip_diversion.c	2021-10-11 19:37:25.000000000 +0100
+++ /usr/src/asterisk/asterisk-16.21.1/res/res_pjsip_diversion.c	2021-10-24 22:52:20.851455767 +0100
@@ -501,34 +501,42 @@
  * \param tdata The outbound message
  * \param data The redirecting data used to fill parts of the diversion header
  */
-static void add_diversion_header(pjsip_tx_data *tdata, struct ast_party_redirecting *data)
+static void add_diversion_header(pjsip_tx_data *tdata, struct ast_sip_session *session)
 {
 	static const pj_str_t reason_name = { "reason", 6 };
+	static const pj_str_t privacy_name = { "privacy", 7 };
 
 	pjsip_fromto_hdr *hdr;
 	pjsip_name_addr *name_addr;
 	pjsip_sip_uri *uri;
 	pjsip_param *param;
+	pjsip_param *privacy;
 	pjsip_fromto_hdr *old_hdr;
 	const char *reason_str;
 	const char *quote_str;
+	const char *privacy_str;
 	char *reason_buf;
+	char *privacy_buf;
 	pjsip_uri *base;
+	int restricted;
 
-	struct ast_party_id *id = NULL;
+	struct ast_party_id id;
 	if (tdata->msg->type == PJSIP_REQUEST_MSG) {
-		id = &data->from;
+		id = ast_channel_redirecting_effective_from(session->channel);
 	} else {
 		/* In responses indicate the new destination */
-		id = &data->to;
+		id = ast_channel_redirecting_effective_to(session->channel);
 	}
 
 	base = PJSIP_MSG_FROM_HDR(tdata->msg)->uri;
 
-	if (!id->number.valid || ast_strlen_zero(id->number.str)) {
+	if (!id.number.valid || ast_strlen_zero(id.number.str)) {
 		return;
 	}
 
+	restricted =
+		((ast_party_id_presentation(&id) & AST_PRES_RESTRICTION) != AST_PRES_ALLOWED);
+
 	hdr = pjsip_from_hdr_create(tdata->pool);
 	hdr->type = PJSIP_H_OTHER;
 	hdr->sname = hdr->name = diversion_name;
@@ -536,24 +544,45 @@
 	name_addr = pjsip_uri_clone(tdata->pool, base);
 	uri = pjsip_uri_get_uri(name_addr->uri);
 
-	pj_strdup2(tdata->pool, &name_addr->display, id->name.str);
-	pj_strdup2(tdata->pool, &uri->user, id->number.str);
+	if (restricted) {
+		privacy_str = "full";
+	} else {
+		privacy_str = "off";
+	}
+
+	if (session->endpoint->id.trust_outbound || !restricted) {
+		pj_strdup2(tdata->pool, &name_addr->display, id.name.str);
+		pj_strdup2(tdata->pool, &uri->user, id.number.str);
+	} else {
+		pj_strdup2(tdata->pool, &name_addr->display, "Anonymous");
+		pj_strdup2(tdata->pool, &uri->user, "anonymous");
+	}
 
 	param = PJ_POOL_ALLOC_T(tdata->pool, pjsip_param);
 	param->name = reason_name;
-
-	reason_str = reason_code_to_str(&data->reason);
+	reason_str = reason_code_to_str(&ast_channel_redirecting(session->channel)->reason);
 
 	/* Reason is either already quoted or it is a token to not need quotes added. */
 	quote_str = *reason_str == '\"' || sip_is_token(reason_str) ? "" : "\"";
 
 	reason_buf = pj_pool_alloc(tdata->pool, strlen(reason_str) + 3);
 	sprintf(reason_buf, "%s%s%s", quote_str, reason_str, quote_str);/* Safe */
-
 	param->value = pj_str(reason_buf);
-
+	
 	pj_list_insert_before(&hdr->other_param, param);
 
+	privacy = PJ_POOL_ALLOC_T(tdata->pool, pjsip_param);
+        privacy->name = privacy_name;
+	
+	/* Privacy is either already quoted or it is a token to not need quotes added. */
+        quote_str = *privacy_str == '\"' || sip_is_token(reason_str) ? "" : "\"";
+
+	privacy_buf = pj_pool_alloc(tdata->pool, strlen(privacy_str) + 3);
+	sprintf(privacy_buf, "%s%s%s", quote_str, privacy_str, quote_str);/* Safe */
+	privacy->value = pj_str(privacy_buf);
+	
+	pj_list_insert_before(&hdr->other_param, privacy);
+
 	hdr->uri = (pjsip_uri *) name_addr;
 	old_hdr = pjsip_msg_find_hdr_by_name(tdata->msg, &diversion_name, NULL);
 	if (old_hdr) {
@@ -656,7 +685,7 @@
 
 	if (session->channel && session->endpoint->id.send_diversion &&
 	    (data = ast_channel_redirecting(session->channel))->count) {
-		add_diversion_header(tdata, data);
+		add_diversion_header(tdata, session);
 	}
 	if (session->channel && session->endpoint->id.send_history_info) {
 		data = ast_channel_redirecting(session->channel);
